#ifndef _IPT_SCTP_H_
#define _IPT_SCTP_H_

#define IPT_SCTP_SRC_PORTS	        0x01
#define IPT_SCTP_DEST_PORTS	        0x02
#define IPT_SCTP_CHUNK_TYPES		0x04

#define IPT_SCTP_VALID_FLAGS		0x07


struct ipt_sctp_flag_info {
	__u8 chunktype;
	__u8 flag;
	__u8 flag_mask;
};

#define IPT_NUM_SCTP_FLAGS	4

struct ipt_sctp_info {
	__u16 dpts[2];  /* Min, Max */
	__u16 spts[2];  /* Min, Max */

	__u32 chunkmap[256 / sizeof (__u32)];  /* Bit mask of chunks to be matched according to RFC 2960 */

#define SCTP_CHUNK_MATCH_ANY   0x01  /* Match if any of the chunk types are present */
#define SCTP_CHUNK_MATCH_ALL   0x02  /* Match if all of the chunk types are present */
#define SCTP_CHUNK_MATCH_ONLY  0x04  /* Match if these are the only chunk types present */

	__u32 chunk_match_type;
	struct ipt_sctp_flag_info flag_info[IPT_NUM_SCTP_FLAGS];
	int flag_count;

	__u32 flags;
	__u32 invflags;
};

#define bytes(type) (sizeof(type) * 8)

#define SCTP_CHUNKMAP_SET(chunkmap, type) 		\
	do { 						\
		chunkmap[type / bytes(__u32)] |= 	\
			1 << (type % bytes(__u32));	\
	} while (0)

#define SCTP_CHUNKMAP_CLEAR(chunkmap, type)		 	\
	do {							\
		chunkmap[type / bytes(__u32)] &= 		\
			~(1 << (type % bytes(__u32)));	\
	} while (0)

#define SCTP_CHUNKMAP_IS_SET(chunkmap, type) 			\
({								\
	(chunkmap[type / bytes (__u32)] & 			\
		(1 << (type % bytes (__u32)))) ? 1: 0;	\
})

#define SCTP_CHUNKMAP_RESET(chunkmap) 				\
	do {							\
		int i; 						\
		for (i = 0; i < ARRAY_SIZE(chunkmap); i++)	\
			chunkmap[i] = 0;			\
	} while (0)

#define SCTP_CHUNKMAP_SET_ALL(chunkmap) 			\
	do {							\
		int i; 						\
		for (i = 0; i < ARRAY_SIZE(chunkmap); i++)	\
			chunkmap[i] = ~0;			\
	} while (0)

#define SCTP_CHUNKMAP_COPY(destmap, srcmap) 			\
	do {							\
		int i; 						\
		for (i = 0; i < ARRAY_SIZE(chunkmap); i++)	\
			destmap[i] = srcmap[i];			\
	} while (0)

#define SCTP_CHUNKMAP_IS_CLEAR(chunkmap) 		\
({							\
	int i; 						\
	int flag = 1;					\
	for (i = 0; i < ARRAY_SIZE(chunkmap); i++) {	\
		if (chunkmap[i]) {			\
			flag = 0;			\
			break;				\
		}					\
	}						\
        flag;						\
})

#define SCTP_CHUNKMAP_IS_ALL_SET(chunkmap) 		\
({							\
	int i; 						\
	int flag = 1;					\
	for (i = 0; i < ARRAY_SIZE(chunkmap); i++) {	\
		if (chunkmap[i] != ~0) {		\
			flag = 0;			\
				break;			\
		}					\
	}						\
        flag;						\
})

#endif /* _IPT_SCTP_H_ */

